@@ -21,15 +21,14 @@
 #include "qgis_core.h"
 
 #include "qgsrasterlayer.h"
-#include "qgsmessagelog.h"
-#include "qgsspatialindex.h"
-#include "qgsprocessing.h"
 #include "qgsfeaturesink.h"
 #include "qgsfeaturesource.h"
+#include "qgsprocessing.h"
 #include "qgsproxyfeaturesink.h"
 #include "qgsremappingproxyfeaturesink.h"
 
 class QgsMeshLayer;
+class QgsPluginLayer;
 class QgsProject;
 class QgsProcessingContext;
 class QgsMapLayerStore;
@@ -37,6 +36,10 @@ class QgsProcessingFeedback;
 class QgsProcessingFeatureSource;
 class QgsProcessingAlgorithm;
 class QgsVectorTileLayer;
+class QgsPointCloudLayer;
+class QgsAnnotationLayer;
+class QgsVectorTileLayer;
+class QgsTiledSceneLayer;
 
 #include <QString>
 #include <QVariant>
@@ -44,7 +47,7 @@ class QgsVectorTileLayer;
 /**
  * \class QgsProcessingUtils
  * \ingroup core
- * Utility functions for use with processing classes.
+ * \brief Utility functions for use with processing classes.
  * \since QGIS 3.0
  */
 class CORE_EXPORT QgsProcessingUtils
@@ -59,6 +62,11 @@ class CORE_EXPORT QgsProcessingUtils
      * value.
      * \see compatibleVectorLayers()
      * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleTiledSceneLayers()
      * \see compatibleLayers()
      */
     static QList< QgsRasterLayer * > compatibleRasterLayers( QgsProject *project, bool sort = true );
@@ -76,6 +84,11 @@ class CORE_EXPORT QgsProcessingUtils
      * value.
      * \see compatibleRasterLayers()
      * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleTiledSceneLayers()
      * \see compatibleLayers()
      */
     static QList< QgsVectorLayer * > compatibleVectorLayers( QgsProject *project,
@@ -91,12 +104,117 @@ class CORE_EXPORT QgsProcessingUtils
      *
      * \see compatibleRasterLayers()
      * \see compatibleVectorLayers()
+     * \see compatiblePluginLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleTiledSceneLayers()
      * \see compatibleLayers()
      *
      * \since QGIS 3.6
      */
     static QList<QgsMeshLayer *> compatibleMeshLayers( QgsProject *project, bool sort = true );
 
+    /**
+     * Returns a list of plugin layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleTiledSceneLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.22
+     */
+    static QList<QgsPluginLayer *> compatiblePluginLayers( QgsProject *project, bool sort = true );
+
+    /**
+     * Returns a list of point cloud layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleTiledSceneLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.22
+     */
+    static QList<QgsPointCloudLayer *> compatiblePointCloudLayers( QgsProject *project, bool sort = true );
+
+    /**
+     * Returns a list of annotation layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleTiledSceneLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.22
+     */
+    static QList<QgsAnnotationLayer *> compatibleAnnotationLayers( QgsProject *project, bool sort = true );
+
+    /**
+     * Returns a list of vector tile layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleTiledSceneLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.32
+     */
+    static QList<QgsVectorTileLayer *> compatibleVectorTileLayers( QgsProject *project, bool sort = true );
+
+    /**
+     * Returns a list of tiled scene layers from a \a project which are compatible with the processing
+     * framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     *
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \see compatiblePointCloudLayers()
+     * \see compatibleAnnotationLayers()
+     * \see compatibleVectorTileLayers()
+     * \see compatibleLayers()
+     *
+     * \since QGIS 3.34
+     */
+    static QList<QgsTiledSceneLayer *> compatibleTiledSceneLayers( QgsProject *project, bool sort = true );
+
     /**
      * Returns a list of map layers from a \a project which are compatible with the processing
      * framework.
@@ -139,6 +257,10 @@ class CORE_EXPORT QgsProcessingUtils
       Vector, //!< Vector layer type
       Raster, //!< Raster layer type
       Mesh, //!< Mesh layer type, since QGIS 3.6
+      PointCloud, //!< Point cloud layer type, since QGIS 3.22
+      Annotation, //!< Annotation layer type, since QGIS 3.22
+      VectorTile, //!< Vector tile layer type, since QGIS 3.32
+      TiledScene, //!< Tiled scene layer type, since QGIS 3.34
     };
 
     /**
@@ -153,7 +275,7 @@ class CORE_EXPORT QgsProcessingUtils
      *
      * The \a typeHint can be used to dictate the type of map layer expected.
      */
-    static QgsMapLayer *mapLayerFromString( const QString &string, QgsProcessingContext &context, bool allowLoadingNewLayers = true, QgsProcessingUtils::LayerHint typeHint = QgsProcessingUtils::LayerHint::UnknownType );
+    static QgsMapLayer *mapLayerFromString( const QString &string, QgsProcessingContext &context, bool allowLoadingNewLayers = true, QgsProcessingUtils::LayerHint typeHint = QgsProcessingUtils::LayerHint::UnknownType, QgsProcessing::LayerOptionsFlags flags = QgsProcessing::LayerOptionsFlags() );
 
     /**
      * Converts a variant \a value to a new feature source.
@@ -204,7 +326,7 @@ class CORE_EXPORT QgsProcessingUtils
      * URI for the resultant layer. It may be updated in place to reflect the actual destination
      * for the layer.
      *
-     * Sink parameters such as desired \a encoding, \a fields, \a geometryType and \a crs must be specified.
+     * Sink parameters such as desired \a fields, \a geometryType and \a crs must be specified.
      *
      * The \a createOptions map can be used to specify additional sink creation options, which
      * are passed to the underlying provider when creating new layers. Known options also
@@ -220,7 +342,7 @@ class CORE_EXPORT QgsProcessingUtils
     static QgsFeatureSink *createFeatureSink( QString &destination,
         QgsProcessingContext &context,
         const QgsFields &fields,
-        QgsWkbTypes::Type geometryType,
+        Qgis::WkbType geometryType,
         const QgsCoordinateReferenceSystem &crs,
         const QVariantMap &createOptions = QVariantMap(),
         const QStringList &datasourceOptions = QStringList(),
@@ -248,7 +370,7 @@ class CORE_EXPORT QgsProcessingUtils
      * SIP bindings. c++ code should call the other createFeatureSink() version.
      * \note available in Python bindings as createFeatureSink()
      */
-    static void createFeatureSinkPython( QgsFeatureSink **sink SIP_OUT SIP_TRANSFERBACK, QString &destination SIP_INOUT, QgsProcessingContext &context, const QgsFields &fields, QgsWkbTypes::Type geometryType, const QgsCoordinateReferenceSystem &crs, const QVariantMap &createOptions = QVariantMap() ) SIP_THROW( QgsProcessingException ) SIP_PYNAME( createFeatureSink );
+    static void createFeatureSinkPython( QgsFeatureSink **sink SIP_OUT SIP_TRANSFERBACK, QString &destination SIP_INOUT, QgsProcessingContext &context, const QgsFields &fields, Qgis::WkbType geometryType, const QgsCoordinateReferenceSystem &crs, const QVariantMap &createOptions = QVariantMap() ) SIP_THROW( QgsProcessingException ) SIP_PYNAME( createFeatureSink );
 
 
     /**
@@ -278,7 +400,7 @@ class CORE_EXPORT QgsProcessingUtils
      * Returns a session specific processing temporary folder for use in processing algorithms.
      * \see generateTempFilename()
      */
-    static QString tempFolder();
+    static QString tempFolder( const QgsProcessingContext *context = nullptr );
 
     /**
      * Returns a temporary filename for a given file, putting it into
@@ -286,7 +408,7 @@ class CORE_EXPORT QgsProcessingUtils
      * but not changing the \a basename.
      * \see tempFolder()
      */
-    static QString generateTempFilename( const QString &basename );
+    static QString generateTempFilename( const QString &basename, const QgsProcessingContext *context = nullptr );
 
     /**
      * Returns a HTML formatted version of the help text encoded in a variant \a map for
@@ -308,6 +430,9 @@ class CORE_EXPORT QgsProcessingUtils
      *
      * The \a featureLimit argument can be used to specify a limit on the number of features read from the layer.
      *
+     * Since QGIS 3.32, the optional \a filterExpression argument can be used to specify a expression to use
+     * to filter the features read from the layer.
+     *
      * When an algorithm is capable of handling multi-layer input files (such as Geopackage), it is preferable
      * to use convertToCompatibleFormatAndLayerName() which may avoid conversion in more situations.
      *
@@ -319,7 +444,9 @@ class CORE_EXPORT QgsProcessingUtils
         const QStringList &compatibleFormats,
         const QString &preferredFormat,
         QgsProcessingContext &context,
-        QgsProcessingFeedback *feedback, long long featureLimit = -1 );
+        QgsProcessingFeedback *feedback,
+        long long featureLimit = -1,
+        const QString &filterExpression = QString() );
 
     /**
      * Converts a source vector \a layer to a file path and layer name of a vector layer of compatible format.
@@ -348,6 +475,7 @@ class CORE_EXPORT QgsProcessingUtils
      * \param feedback feedback object
      * \param layerName will be set to the target layer name for multi-layer sources (e.g. Geopackage)
      * \param featureLimit can be used to place a limit on the maximum number of features read from the layer
+     * \param filterExpression optional expression for filtering features read from the layer (since QGIS 3.32)
      *
      * \returns path to source layer, or nearly converted compatible layer
      *
@@ -361,7 +489,9 @@ class CORE_EXPORT QgsProcessingUtils
         const QString &preferredFormat,
         QgsProcessingContext &context,
         QgsProcessingFeedback *feedback,
-        QString &layerName SIP_OUT, long long featureLimit = -1 );
+        QString &layerName SIP_OUT,
+        long long featureLimit = -1,
+        const QString &filterExpression = QString() );
 
     /**
      * Combines two field lists, avoiding duplicate field names (in a case-insensitive manner).
@@ -396,6 +526,8 @@ class CORE_EXPORT QgsProcessingUtils
      * a fallback value of "gpkg".
      *
      * \see defaultRasterExtension()
+     * \see defaultPointCloudExtension()
+     * \see defaultVectorTileExtension()
      * \since QGIS 3.10
      */
     static QString defaultVectorExtension();
@@ -408,17 +540,90 @@ class CORE_EXPORT QgsProcessingUtils
      * a fallback value of "tif".
      *
      * \see defaultVectorExtension()
+     * \see defaultPointCloudExtension()
+     * \see defaultVectorTileExtension()
      * \since QGIS 3.10
      */
     static QString defaultRasterExtension();
 
+    /**
+     * Returns the default point cloud extension to use, in the absence of all other constraints (e.g.
+     * provider based support for extensions).
+     *
+     * This method returns a fallback value of "las".
+     *
+     * \see defaultVectorExtension()
+     * \see defaultRasterExtension()
+     * \see defaultVectorTileExtension()
+     * \since QGIS 3.24
+     */
+    static QString defaultPointCloudExtension();
+
+    /**
+     * Returns the default vector tile extension to use, in the absence of all other constraints (e.g.
+     * provider based support for extensions).
+     *
+     * This method returns a fallback value of "mbtiles".
+     *
+     * \see defaultVectorExtension()
+     * \see defaultRasterExtension()
+     * \see defaultPointCloudExtension()
+     * \since QGIS 3.32
+     */
+    static QString defaultVectorTileExtension();
+
+    /**
+     * Removes any raw pointer values from an input \a map, replacing them with
+     * appropriate string values where possible.
+     *
+     * \since QGIS 3.26
+     */
+    static QVariantMap removePointerValuesFromMap( const QVariantMap &map );
+
+    /**
+     * Pre-processes a set of \a parameter values for the qgis_process command.
+     *
+     * In particular, this function upgrades encoded data defined values to QgsProperty objects.
+     *
+     * \since QGIS 3.30
+     */
+    static QVariantMap preprocessQgisProcessParameters( const QVariantMap &parameters, bool &ok, QString &error );
+
+    /**
+     * Returns the default encoding.
+     *
+     * The default encoding could be the one from "/Processing/encoding" or when it's not an allowed encoding name
+     * like "System", the default encoding system (mostly UTF-8 on Unix-like, windows-1252 on Windows).
+     *
+     * \since QGIS 3.32
+     */
+    static QString resolveDefaultEncoding( const QString &defaultEncoding = "System" );
+
   private:
     static bool canUseLayer( const QgsRasterLayer *layer );
     static bool canUseLayer( const QgsMeshLayer *layer );
+    static bool canUseLayer( const QgsPluginLayer *layer );
     static bool canUseLayer( const QgsVectorTileLayer *layer );
+    static bool canUseLayer( const QgsPointCloudLayer *layer );
+    static bool canUseLayer( const QgsAnnotationLayer *layer );
+    static bool canUseLayer( const QgsTiledSceneLayer *layer );
     static bool canUseLayer( const QgsVectorLayer *layer,
                              const QList< int > &sourceTypes = QList< int >() );
 
+    /**
+     * Returns a list of map layers with the given layer type from a \a project which are compatible
+     * with the processing framework.
+     *
+     * If the \a sort argument is TRUE then the layers will be sorted by their QgsMapLayer::name()
+     * value.
+     * \see compatibleRasterLayers()
+     * \see compatibleVectorLayers()
+     * \see compatibleMeshLayers()
+     * \see compatiblePluginLayers()
+     * \since QGIS 3.22
+     */
+    template< typename T> static QList< T * > compatibleMapLayers( QgsProject *project, bool sort = true );
+
     /**
      * Interprets a \a string as a map layer from a store.
      *
@@ -441,7 +646,7 @@ class CORE_EXPORT QgsProcessingUtils
      *
      * \since QGIS 3.8
      */
-    static QgsMapLayer *loadMapLayerFromString( const QString &string, const QgsCoordinateTransformContext &transformContext, LayerHint typeHint = LayerHint::UnknownType );
+    static QgsMapLayer *loadMapLayerFromString( const QString &string, const QgsCoordinateTransformContext &transformContext, LayerHint typeHint = LayerHint::UnknownType, QgsProcessing::LayerOptionsFlags flags = QgsProcessing::LayerOptionsFlags() );
 
     /**
      * Interprets a string as a map layer. The method will attempt to
@@ -463,7 +668,7 @@ class CORE_EXPORT QgsProcessingUtils
 /**
  * \class QgsProcessingFeatureSource
  * \ingroup core
- * QgsFeatureSource subclass which proxies methods to an underlying QgsFeatureSource, modifying
+ * \brief QgsFeatureSource subclass which proxies methods to an underlying QgsFeatureSource, modifying
  * results according to the settings in a QgsProcessingContext.
  * \since QGIS 3.0
  */
@@ -487,9 +692,11 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
      *
      * If \a featureLimit is set to a value > 0, then a limit is placed on the maximum number of features which will be
      * read from the source.
+     *
+     * Since QGIS 3.32, the optional \a filterExpression can be used to specify an expression based filter for the source.
      */
     QgsProcessingFeatureSource( QgsFeatureSource *originalSource, const QgsProcessingContext &context, bool ownsOriginalSource = false,
-                                long long featureLimit = -1 );
+                                long long featureLimit = -1, const QString &filterExpression = QString() );
 
     ~QgsProcessingFeatureSource() override;
 
@@ -505,8 +712,8 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     QgsFeatureIterator getFeatures( const QgsFeatureRequest &request = QgsFeatureRequest() ) const override;
     QgsCoordinateReferenceSystem sourceCrs() const override;
     QgsFields fields() const override;
-    QgsWkbTypes::Type wkbType() const override;
-    long featureCount() const override;
+    Qgis::WkbType wkbType() const override;
+    long long featureCount() const override;
     QString sourceName() const override;
     QSet<QVariant> uniqueValues( int fieldIndex, int limit = -1 ) const override;
     QVariant minimumValue( int fieldIndex ) const override;
@@ -531,6 +738,13 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
 
     QgsFeatureSource *mSource = nullptr;
     bool mOwnsSource = false;
+    QgsCoordinateReferenceSystem mSourceCrs;
+    QgsFields mSourceFields;
+    Qgis::WkbType mSourceWkbType = Qgis::WkbType::Unknown;
+    QString mSourceName;
+    QgsRectangle mSourceExtent;
+    QgsFeatureSource::SpatialIndexPresence mSourceSpatialIndexPresence = QgsFeatureSource::SpatialIndexPresence::SpatialIndexUnknown;
+
     QgsFeatureRequest::InvalidGeometryCheck mInvalidGeometryCheck = QgsFeatureRequest::GeometryNoCheck;
     std::function< void( const QgsFeature & ) > mInvalidGeometryCallback;
     std::function< void( const QgsFeature & ) > mTransformErrorCallback;
@@ -539,6 +753,7 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
     std::function< void( const QgsFeature & ) > mInvalidGeometryCallbackAbort;
 
     long long mFeatureLimit = -1;
+    QString mFilterExpression;
 
 };
 
@@ -547,7 +762,7 @@ class CORE_EXPORT QgsProcessingFeatureSource : public QgsFeatureSource
 /**
  * \class QgsProcessingFeatureSink
  * \ingroup core
- * QgsProxyFeatureSink subclass which reports feature addition errors to a QgsProcessingContext.
+ * \brief QgsProxyFeatureSink subclass which reports feature addition errors to a QgsProcessingContext.
  * \note Not available in Python bindings.
  * \since QGIS 3.0
  */
