@@ -26,7 +26,7 @@
 
 class QString;
 class QRectF;
-class QgsBox3d;
+class QgsBox3D;
 #include "qgspointxy.h"
 
 
@@ -36,7 +36,7 @@ class QgsBox3d;
  *
  * QgsRectangle is used to store a rectangle when double values are required.
  * Examples are storing a layer extent or the current view extent of a map
- * \see QgsBox3d
+ * \see QgsBox3D
  */
 class CORE_EXPORT QgsRectangle
 {
@@ -165,16 +165,31 @@ class CORE_EXPORT QgsRectangle
      */
     void setYMaximum( double y ) SIP_HOLDGIL { mYmax = y; }
 
+    /**
+     * Mark a rectangle as being null (holding no spatial information).
+     *
+     * A null rectangle is also empty by definition.
+     *
+     * \see isNull()
+     * \see isEmpty()
+     *
+     * \since QGIS 3.34
+     */
+    void setNull() SIP_HOLDGIL
+    {
+      mXmin = mYmin = std::numeric_limits<double>::max();
+      mXmax = mYmax = -std::numeric_limits<double>::max();
+    }
+
     /**
      * Set a rectangle so that min corner is at max
      * and max corner is at min. It is NOT normalized.
+     *
+     * \deprecated since QGIS 3.34 - will be removed in QGIS 4.0. Use setNull().
      */
-    void setMinimal() SIP_HOLDGIL
+    Q_DECL_DEPRECATED void setMinimal() SIP_DEPRECATED
     {
-      mXmin = std::numeric_limits<double>::max();
-      mYmin = std::numeric_limits<double>::max();
-      mXmax = -std::numeric_limits<double>::max();
-      mYmax = -std::numeric_limits<double>::max();
+      setNull();
     }
 
     /**
@@ -295,6 +310,8 @@ class CORE_EXPORT QgsRectangle
      */
     void grow( double delta )
     {
+      if ( isNull() )
+        return;
       mXmin -= delta;
       mXmax += delta;
       mYmin -= delta;
@@ -306,6 +323,14 @@ class CORE_EXPORT QgsRectangle
      */
     void include( const QgsPointXY &p )
     {
+      if ( isNull() )
+      {
+        setXMinimum( p.x() );
+        setXMaximum( p.x() );
+        setYMinimum( p.y() );
+        setYMaximum( p.y() );
+        return;
+      }
       if ( p.x() < xMinimum() )
         setXMinimum( p.x() );
       if ( p.x() > xMaximum() )
@@ -324,6 +349,8 @@ class CORE_EXPORT QgsRectangle
     */
     QgsRectangle buffered( double width ) const
     {
+      if ( isNull() )
+        return QgsRectangle();
       return QgsRectangle( mXmin - width, mYmin - width, mXmax + width, mYmax + width );
     }
 
@@ -463,23 +490,33 @@ class CORE_EXPORT QgsRectangle
     QgsRectangle &operator+=( QgsVector v );
 
     /**
-     * Returns TRUE if the rectangle is empty.
-     * An empty rectangle may still be non-null if it contains valid information (e.g. bounding box of a point).
+     * Returns TRUE if the rectangle has no area.
+     *
+     * An empty rectangle may still be non-null if it contains valid
+     * spatial information (e.g. bounding box of a point or of a vertical
+     * or horizontal segment).
+     *
+     * \see isNull()
      */
     bool isEmpty() const
     {
-      return mXmax < mXmin || mYmax < mYmin || qgsDoubleNear( mXmax, mXmin ) || qgsDoubleNear( mYmax, mYmin );
+      return isNull() || mXmax <= mXmin || mYmax <= mYmin || qgsDoubleNear( mXmax, mXmin ) || qgsDoubleNear( mYmax, mYmin );
     }
 
     /**
-     * Test if the rectangle is null (all coordinates zero or after call to setMinimal()).
+     * Test if the rectangle is null (holding no spatial information).
+     *
      * A null rectangle is also an empty rectangle.
+     *
+     * \see setNull()
+     *
      * \since QGIS 2.4
      */
     bool isNull() const
     {
-      // rectangle created QgsRectangle() or with rect.setMinimal() ?
-      return ( qgsDoubleNear( mXmin, 0.0 ) && qgsDoubleNear( mXmax, 0.0 ) && qgsDoubleNear( mYmin, 0.0 ) && qgsDoubleNear( mYmax, 0.0 ) ) ||
+      // rectangle created QgsRectangle() or with rect.setNull() or
+      // otherwise having NaN ordinates
+      return ( std::isnan( mXmin )  && std::isnan( mXmax ) && std::isnan( mYmin ) && std::isnan( mYmax ) ) ||
              ( qgsDoubleNear( mXmin, std::numeric_limits<double>::max() ) && qgsDoubleNear( mYmin, std::numeric_limits<double>::max() ) &&
                qgsDoubleNear( mXmax, -std::numeric_limits<double>::max() ) && qgsDoubleNear( mYmax, -std::numeric_limits<double>::max() ) );
     }
@@ -520,6 +557,8 @@ class CORE_EXPORT QgsRectangle
      */
     bool operator==( const QgsRectangle &r1 ) const
     {
+      if ( isNull() ) return r1.isNull();
+
       return qgsDoubleNear( r1.xMaximum(), xMaximum() ) &&
              qgsDoubleNear( r1.xMinimum(), xMinimum() ) &&
              qgsDoubleNear( r1.yMaximum(), yMaximum() ) &&
@@ -583,7 +622,7 @@ class CORE_EXPORT QgsRectangle
      * \a zMin and \a zMax z values.
      * \since QGIS 3.0
      */
-    QgsBox3d toBox3d( double zMin, double zMax ) const;
+    QgsBox3D toBox3d( double zMin, double zMax ) const;
 
     //! Allows direct construction of QVariants from rectangles.
     operator QVariant() const
@@ -609,10 +648,10 @@ class CORE_EXPORT QgsRectangle
 
   private:
 
-    double mXmin = 0.0;
-    double mYmin = 0.0;
-    double mXmax = 0.0;
-    double mYmax = 0.0;
+    double mXmin = std::numeric_limits<double>::max();
+    double mYmin = std::numeric_limits<double>::max();
+    double mXmax = -std::numeric_limits<double>::max();
+    double mYmax = -std::numeric_limits<double>::max();
 
 };
 
