@@ -20,9 +20,9 @@
 #include "qgis_sip.h"
 #include "qgis.h"
 #include "qgsrectangle.h"
-#include "qgsrendercontext.h"
 #include "qgsfields.h"
 #include "qgsfeaturerequest.h"
+#include "qgsconfig.h"
 
 #include <QList>
 #include <QString>
@@ -37,6 +37,7 @@ class QgsVectorLayer;
 class QgsPaintEffect;
 class QgsReadWriteContext;
 class QgsStyleEntityVisitorInterface;
+class QgsRenderContext;
 
 typedef QMap<QString, QString> QgsStringMap SIP_SKIP;
 
@@ -136,7 +137,7 @@ class CORE_EXPORT QgsFeatureRenderer
     // renderer takes ownership of its symbols!
 
     //! Returns a new renderer - used by default in vector layers
-    static QgsFeatureRenderer *defaultRenderer( QgsWkbTypes::GeometryType geomType ) SIP_FACTORY;
+    static QgsFeatureRenderer *defaultRenderer( Qgis::GeometryType geomType ) SIP_FACTORY;
 
     QString type() const { return mType; }
 
@@ -191,6 +192,16 @@ class CORE_EXPORT QgsFeatureRenderer
      */
     virtual void stopRender( QgsRenderContext &context );
 
+    /**
+     * Returns TRUE if the renderer can be entirely skipped, i.e. if it is known in advance
+     * that no features will be rendered.
+     *
+     * \warning Must be called between startRender() and stopRender() calls.
+     *
+     * \since QGIS 3.30
+     */
+    virtual bool canSkipRender();
+
     /**
      * If a renderer does not require all the features this method may be overridden
      * and return an expression used as where clause.
@@ -294,7 +305,12 @@ class CORE_EXPORT QgsFeatureRenderer
     //! create a renderer from XML element
     static QgsFeatureRenderer *load( QDomElement &symbologyElem, const QgsReadWriteContext &context ) SIP_FACTORY;
 
-    //! store renderer info to XML element
+    /**
+     * Stores renderer properties to an XML element.
+     *
+     * Subclasses which override this method should call saveRendererData() as part of their
+     * implementation in order to store all common base class properties in the returned DOM element.
+     */
     virtual QDomElement save( QDomDocument &doc, const QgsReadWriteContext &context );
 
     /**
@@ -314,7 +330,7 @@ class CORE_EXPORT QgsFeatureRenderer
      * went wrong
      * \returns the renderer
      */
-    static QgsFeatureRenderer *loadSld( const QDomNode &node, QgsWkbTypes::GeometryType geomType, QString &errorMessage ) SIP_FACTORY;
+    static QgsFeatureRenderer *loadSld( const QDomNode &node, Qgis::GeometryType geomType, QString &errorMessage ) SIP_FACTORY;
 
     //! used from subclasses to create SLD Rule elements following SLD v1.1 specs
     virtual void toSld( QDomDocument &doc, QDomElement &element, const QVariantMap &props = QVariantMap() ) const
@@ -324,19 +340,37 @@ class CORE_EXPORT QgsFeatureRenderer
     }
 
     /**
-     * items of symbology items in legend should be checkable
+     * Returns the set of all legend keys used by the renderer.
+     *
+     * \see legendSymbolItems()
+     *
+     * \since QGIS 3.32
+     */
+    QSet< QString > legendKeys() const;
+
+    /**
+     * Returns TRUE if symbology items in legend are checkable.
+     *
      * \since QGIS 2.5
      */
     virtual bool legendSymbolItemsCheckable() const;
 
     /**
-     * items of symbology items in legend is checked
+     * Returns TRUE if the legend symbology item with the specified \a key is checked.
+     *
+     * \see checkLegendSymbolItem()
+     * \see legendKeys()
+     *
      * \since QGIS 2.5
      */
     virtual bool legendSymbolItemChecked( const QString &key );
 
     /**
-     * item in symbology was checked
+     * Sets whether the legend symbology item with the specified \a ley should be checked.
+     *
+     * \see legendSymbolItemChecked()
+     * \see legendKeys()
+     *
      * \since QGIS 2.5
      */
     virtual void checkLegendSymbolItem( const QString &key, bool state = true );
@@ -345,12 +379,34 @@ class CORE_EXPORT QgsFeatureRenderer
      * Sets the symbol to be used for a legend symbol item.
      * \param key rule key for legend symbol
      * \param symbol new symbol for legend item. Ownership is transferred to renderer.
+     *
+     * \see legendKeys()
+     *
      * \since QGIS 2.14
      */
     virtual void setLegendSymbolItem( const QString &key, QgsSymbol *symbol SIP_TRANSFER );
 
+    /**
+     * Attempts to convert the specified legend rule \a key to a QGIS expression matching
+     * the features displayed using that key.
+     *
+     * \param key legend key
+     * \param layer associated vector layer
+     * \param ok will be set to TRUE if legend key was successfully converted to a filter expression
+     *
+     * \returns QGIS expression string for matching features with the specified key
+     *
+     * \see legendKeys()
+     *
+     * \since QGIS 3.26
+     */
+    virtual QString legendKeyToExpression( const QString &key, QgsVectorLayer *layer, bool &ok SIP_OUT ) const;
+
     /**
      * Returns a list of symbology items for the legend
+     *
+     * \see legendKeys()
+     *
      * \since QGIS 2.6
      */
     virtual QgsLegendSymbolList legendSymbolItems() const;
@@ -362,7 +418,7 @@ class CORE_EXPORT QgsFeatureRenderer
     virtual QString legendClassificationAttribute() const { return QString(); }
 
     //! Sets type and size of editing vertex markers for subsequent rendering
-    void setVertexMarkerAppearance( int type, double size );
+    void setVertexMarkerAppearance( Qgis::VertexMarkerType type, double size );
 
     /**
      * Returns whether the renderer will render a feature or not.
@@ -410,7 +466,7 @@ class CORE_EXPORT QgsFeatureRenderer
      * \see paintEffect
      * \since QGIS 2.9
      */
-    void setPaintEffect( QgsPaintEffect *effect );
+    void setPaintEffect( QgsPaintEffect *effect SIP_TRANSFER );
 
     /**
      * Returns whether the renderer must render as a raster.
@@ -429,6 +485,38 @@ class CORE_EXPORT QgsFeatureRenderer
      */
     void setForceRasterRender( bool forceRaster ) { mForceRaster = forceRaster; }
 
+    /**
+     * Returns the symbology reference scale.
+     *
+     * This represents the desired scale denominator for the rendered map, eg 1000.0 for a 1:1000 map render.
+     * A value of -1 indicates that symbology scaling by reference scale is disabled.
+     *
+     * The symbology reference scale is an optional property which specifies the reference
+     * scale at which symbology in paper units (such a millimeters or points) is fixed
+     * to. For instance, if the scale is 1000 then a 2mm thick line will be rendered at
+     * exactly 2mm thick when a map is rendered at 1:1000, or 1mm thick when rendered at 1:2000, or 4mm thick at 1:500.
+     *
+     * \see setReferenceScale()
+     * \since QGIS 3.22
+     */
+    double referenceScale() const { return mReferenceScale; }
+
+    /**
+     * Sets the symbology reference \a scale.
+     *
+     * This should match the desired scale denominator for the rendered map, eg 1000.0 for a 1:1000 map render.
+     * Set to -1 to disable symbology scaling by reference scale.
+     *
+     * The symbology reference scale is an optional property which specifies the reference
+     * scale at which symbology in paper units (such a millimeters or points) is fixed
+     * to. For instance, if \a scale is set to 1000 then a 2mm thick line will be rendered at
+     * exactly 2mm thick when a map is rendered at 1:1000, or 1mm thick when rendered at 1:2000, or 4mm thick at 1:500.
+     *
+     * \see referenceScale()
+     * \since QGIS 3.22
+     */
+    void setReferenceScale( double scale ) { mReferenceScale = scale; }
+
     /**
      * Gets the order in which features shall be processed by this renderer.
      * \note this property has no effect if orderByEnabled() is FALSE
@@ -490,6 +578,22 @@ class CORE_EXPORT QgsFeatureRenderer
      */
     virtual bool accept( QgsStyleEntityVisitorInterface *visitor ) const;
 
+    /**
+     * Clones generic renderer data to another renderer.
+     *
+     * Currently clones
+     *
+     * - Order by
+     * - Paint effect
+     * - Reference scale
+     * - Symbol levels enabled/disabled
+     * - Force raster render enabled/disabled
+     *
+     * \param destRenderer destination renderer for copied effect
+     * \since QGIS 3.22
+     */
+    void copyRendererData( QgsFeatureRenderer *destRenderer ) const;
+
   protected:
     QgsFeatureRenderer( const QString &type );
 
@@ -515,22 +619,22 @@ class CORE_EXPORT QgsFeatureRenderer
     static QPointF _getPoint( QgsRenderContext &context, const QgsPoint &point );
 
     /**
-     * Clones generic renderer data to another renderer.
-     * Currently clones
+     * Saves generic renderer data into the specified \a element.
      *
-     * - Order By
-     * - Paint Effect
+     * This method should be called in a subclass' save() implementation in order
+     * to store all common base class properties in the DOM \a element.
      *
-     * \param destRenderer destination renderer for copied effect
+     * \since QGIS 3.22
      */
-    void copyRendererData( QgsFeatureRenderer *destRenderer ) const;
+    void saveRendererData( QDomDocument &doc, QDomElement &element, const QgsReadWriteContext &context );
 
     QString mType;
 
     bool mUsingSymbolLevels = false;
 
     //! The current type of editing marker
-    int mCurrentVertexMarkerType;
+    Qgis::VertexMarkerType mCurrentVertexMarkerType = Qgis::VertexMarkerType::Cross;
+
     //! The current size of editing marker
     double mCurrentVertexMarkerSize = 2;
 
@@ -538,6 +642,8 @@ class CORE_EXPORT QgsFeatureRenderer
 
     bool mForceRaster = false;
 
+    double mReferenceScale = -1.0;
+
     /**
      * \note this function is used to convert old sizeScale expressions to symbol
      * level DataDefined size
