@@ -196,7 +196,7 @@ class GUI_EXPORT QgsUnitSelectionWidget : public QWidget, private Ui::QgsUnitSel
      * \returns selected output unit, or QgsUnitTypes::RenderUnknownUnit if the widget was populated with custom unit types
      * \since QGIS 2.9
      */
-    QgsUnitTypes::RenderUnit unit() const;
+    Qgis::RenderUnit unit() const;
 
     /**
      * Sets the selected unit index
@@ -209,7 +209,7 @@ class GUI_EXPORT QgsUnitSelectionWidget : public QWidget, private Ui::QgsUnitSel
      * Sets the selected unit
      * \param unit predefined unit to set as current
      */
-    void setUnit( QgsUnitTypes::RenderUnit unit );
+    void setUnit( Qgis::RenderUnit unit );
 
     //! Returns the map unit scale
     QgsMapUnitScale getMapUnitScale() const { return mMapUnitScale; }
@@ -225,6 +225,22 @@ class GUI_EXPORT QgsUnitSelectionWidget : public QWidget, private Ui::QgsUnitSel
      */
     void setMapCanvas( QgsMapCanvas *canvas );
 
+    /**
+     * Returns TRUE if the widget can show the map scale button when the Map Units option is selected.
+     *
+     * \see setShowMapScaleButton()
+     * \since QGIS 3.22
+     */
+    bool showMapScaleButton() const;
+
+    /**
+     * Sets whether the widget can show the map scale button when the Map Units option is selected.
+     *
+     * \see showMapScaleButton()
+     * \since QGIS 3.22
+     */
+    void setShowMapScaleButton( bool show );
+
   signals:
     void changed();
 
@@ -237,6 +253,7 @@ class GUI_EXPORT QgsUnitSelectionWidget : public QWidget, private Ui::QgsUnitSel
     QgsMapUnitScale mMapUnitScale;
     int mMapUnitIdx;
     QgsMapCanvas *mCanvas = nullptr;
+    bool mShowMapScaleButton = true;
 
 };
 
