@@ -23,12 +23,11 @@
 #include "qgis_sip.h"
 #include "qgsfields.h"
 #include "qgsfeedback.h"
-#include "qgssymbol.h"
-#include "qgstaskmanager.h"
 #include "qgsogrutils.h"
 #include "qgsrenderer.h"
 #include "qgsgeometryengine.h"
 #include "qgsfeaturesink.h"
+#include "qgsrendercontext.h"
 #include <ogr_api.h>
 
 class QgsSymbolLayer;
@@ -37,10 +36,12 @@ class QgsFeatureIterator;
 
 /**
  * \ingroup core
-  * A convenience class for writing vector files to disk.
- There are two possibilities how to use this class:
- 1. static call to QgsVectorFileWriter::writeAsVectorFormat(...) which saves the whole vector layer
- 2. create an instance of the class and issue calls to addFeature(...)
+ * \brief A convenience class for writing vector layers to disk based formats (e.g. Shapefiles, GeoPackage).
+ *
+ * There are two possibilities how to use this class:
+ *
+ * 1. A static call to QgsVectorFileWriter::writeAsVectorFormat(...) which saves the whole vector layer.
+ * 2. Create an instance of the class and issue calls to addFeature(...).
  */
 class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 {
@@ -76,7 +77,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       public:
         SetOption( const QString &docString, const QStringList &values, const QString &defaultValue, bool allowNone = false )
           : Option( docString, Set )
-          , values( qgis::listToSet( values ) )
+          , values( values.begin(), values.end() )
           , defaultValue( defaultValue )
           , allowNone( allowNone )
         {}
@@ -175,17 +176,21 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       ErrProjection,
       ErrFeatureWriteFailed,
       ErrInvalidLayer,
+      ErrSavingMetadata, //!< Metadata saving failed
       Canceled, //!< Writing was interrupted by manual cancellation
     };
 
-    enum SymbologyExport
+    /**
+     * Source for exported field names.
+     *
+     * \since QGIS 3.18
+     */
+    enum FieldNameSource
     {
-      NoSymbology = 0, //export only data
-      FeatureSymbology, //Keeps the number of features and export symbology per feature
-      SymbolLayerSymbology //Exports one feature per symbol layer (considering symbol levels)
+      Original = 0, //!< Use original field names
+      PreferAlias, //!< Use the field alias as the exported field name, wherever one is set. Otherwise use the original field names.
     };
 
-
     /**
      * Options for sorting and filtering vector formats.
      * \since QGIS 3.0
@@ -194,12 +199,13 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     {
       SortRecommended = 1 << 1, //!< Use recommended sort order, with extremely commonly used formats listed first
       SkipNonSpatialFormats = 1 << 2, //!< Filter out any formats which do not have spatial support (e.g. those which cannot save geometries)
+      SupportsMultipleLayers = 1 << 3, //!< Filter to only formats which support multiple layers (since QGIS 3.32)
     };
     Q_DECLARE_FLAGS( VectorFormatOptions, VectorFormatOption )
 
     /**
      * \ingroup core
-     * Interface to convert raw field values to their user-friendly value.
+     * \brief Interface to convert raw field values to their user-friendly value.
      * \since QGIS 2.16
      */
     class CORE_EXPORT FieldValueConverter
@@ -233,7 +239,8 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     /**
      * Edition capability flags
-      * \since QGIS 3.0 */
+     * \since QGIS 3.0
+    */
     enum EditionCapability
     {
       //! Flag to indicate that a new layer can be added to the dataset
@@ -251,12 +258,13 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     /**
      * Combination of CanAddNewLayer, CanAppendToExistingLayer, CanAddNewFieldsToExistingLayer or CanDeleteLayer
-      * \since QGIS 3.0 */
+     * \since QGIS 3.0.
+    */
     Q_DECLARE_FLAGS( EditionCapabilities, EditionCapability )
 
     /**
      * Enumeration to describe how to handle existing files
-        \since QGIS 3.0
+     * \since QGIS 3.0
      */
     enum ActionOnExistingFile
     {
@@ -338,10 +346,10 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         const QStringList &layerOptions = QStringList(),
         bool skipAttributeCreation = false,
         QString *newFilename = nullptr,
-        QgsVectorFileWriter::SymbologyExport symbologyExport = QgsVectorFileWriter::NoSymbology,
+        Qgis::FeatureSymbologyExport symbologyExport = Qgis::FeatureSymbologyExport::NoSymbology,
         double symbologyScale = 1.0,
         const QgsRectangle *filterExtent = nullptr,
-        QgsWkbTypes::Type overrideGeometryType = QgsWkbTypes::Unknown,
+        Qgis::WkbType overrideGeometryType = Qgis::WkbType::Unknown,
         bool forceMulti = false,
         bool includeZ = false,
         const QgsAttributeList &attributes = QgsAttributeList(),
@@ -421,10 +429,10 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         const QStringList &layerOptions = QStringList(),
         bool skipAttributeCreation = false,
         QString *newFilename = nullptr,
-        QgsVectorFileWriter::SymbologyExport symbologyExport = QgsVectorFileWriter::NoSymbology,
+        Qgis::FeatureSymbologyExport symbologyExport = Qgis::FeatureSymbologyExport::NoSymbology,
         double symbologyScale = 1.0,
         const QgsRectangle *filterExtent = nullptr,
-        QgsWkbTypes::Type overrideGeometryType = QgsWkbTypes::Unknown,
+        Qgis::WkbType overrideGeometryType = Qgis::WkbType::Unknown,
         bool forceMulti = false,
         bool includeZ = false,
         const QgsAttributeList &attributes = QgsAttributeList(),
@@ -437,7 +445,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     /**
      * \ingroup core
-     * Options to pass to writeAsVectorFormat()
+     * \brief Options to pass to writeAsVectorFormat()
      * \since QGIS 3.0
      */
     class CORE_EXPORT SaveVectorOptions
@@ -462,7 +470,8 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
         /**
          * Transform to reproject exported geometries with, or invalid transform
-         * for no transformation */
+         * for no transformation
+        */
         QgsCoordinateTransform ct;
 
         //! Write only selected features of layer
@@ -480,8 +489,11 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         //! Attributes to export (empty means all unless skipAttributeCreation is set)
         QgsAttributeList attributes;
 
+        //! Attributes export names
+        QStringList attributesExportNames;
+
         //! Symbology to export
-        QgsVectorFileWriter::SymbologyExport symbologyExport = NoSymbology;
+        Qgis::FeatureSymbologyExport symbologyExport = Qgis::FeatureSymbologyExport::NoSymbology;
 
         //! Scale of symbology
         double symbologyScale = 1.0;
@@ -491,8 +503,9 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
         /**
          * Set to a valid geometry type to override the default geometry type for the layer. This parameter
-         * allows for conversion of geometryless tables to null geometries, etc */
-        QgsWkbTypes::Type overrideGeometryType = QgsWkbTypes::Unknown;
+         * allows for conversion of geometryless tables to null geometries, etc.
+        */
+        Qgis::WkbType overrideGeometryType = Qgis::WkbType::Unknown;
 
         //! Sets to TRUE to force creation of multi* geometries
         bool forceMulti = false;
@@ -510,6 +523,38 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
         //! Optional feedback object allowing cancellation of layer save
         QgsFeedback *feedback = nullptr;
+
+        /**
+         * Source for exported field names.
+         *
+         * \since QGIS 3.18
+         */
+        FieldNameSource fieldNameSource = Original;
+
+        /**
+         * Set to TRUE to save layer metadata for the exported vector file.
+         *
+         * \see layerMetadata
+         * \since QGIS 3.20
+         */
+        bool saveMetadata = false;
+
+        /**
+         * Layer metadata to save for the exported vector file. This will only be used if saveMetadata is TRUE.
+         *
+         * \see saveMetadata
+         * \since QGIS 3.20
+         */
+        QgsLayerMetadata layerMetadata;
+
+        /**
+         * Set to TRUE to transfer field constraints to the exported vector file.
+         *
+         * Support for field constraints depends on the output file format.
+         *
+         * \since QGIS 3.34
+         */
+        bool includeConstraints = false;
     };
 
 #ifndef SIP_RUN
@@ -556,17 +601,18 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     Q_DECL_DEPRECATED QgsVectorFileWriter( const QString &vectorFileName,
                                            const QString &fileEncoding,
                                            const QgsFields &fields,
-                                           QgsWkbTypes::Type geometryType,
+                                           Qgis::WkbType geometryType,
                                            const QgsCoordinateReferenceSystem &srs = QgsCoordinateReferenceSystem(),
                                            const QString &driverName = "GPKG",
                                            const QStringList &datasourceOptions = QStringList(),
                                            const QStringList &layerOptions = QStringList(),
                                            QString *newFilename = nullptr,
-                                           QgsVectorFileWriter::SymbologyExport symbologyExport = QgsVectorFileWriter::NoSymbology,
+                                           Qgis::FeatureSymbologyExport symbologyExport = Qgis::FeatureSymbologyExport::NoSymbology,
                                            QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags()
 #ifndef SIP_RUN
                                                , QString *newLayer = nullptr,
-                                           QgsCoordinateTransformContext transformContext = QgsCoordinateTransformContext()
+                                           const QgsCoordinateTransformContext &transformContext = QgsCoordinateTransformContext(),
+                                           FieldNameSource fieldNameSource = Original
 #endif
                                          ) SIP_DEPRECATED;
 
@@ -588,25 +634,29 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * \param newLayer potentially modified layer name (output parameter) (added in QGIS 3.4)
      * \param transformContext transform context, needed if the output file srs is forced to specific crs (added in QGIS 3.10.3)
      * \param sinkFlags feature sink flags (added in QGIS 3.10.3)
+     * \param fieldNameSource source for field names (since QGIS 3.18)
+     * \param includeConstraints set to TRUE to copy field constraints to the destination layer (since QGIS 3.34)
      * \note not available in Python bindings
      * \deprecated Use create() instead.
      */
     Q_DECL_DEPRECATED QgsVectorFileWriter( const QString &vectorFileName,
                                            const QString &fileEncoding,
                                            const QgsFields &fields,
-                                           QgsWkbTypes::Type geometryType,
+                                           Qgis::WkbType geometryType,
                                            const QgsCoordinateReferenceSystem &srs,
                                            const QString &driverName,
                                            const QStringList &datasourceOptions,
                                            const QStringList &layerOptions,
                                            QString *newFilename,
-                                           QgsVectorFileWriter::SymbologyExport symbologyExport,
+                                           Qgis::FeatureSymbologyExport symbologyExport,
                                            QgsVectorFileWriter::FieldValueConverter *fieldValueConverter,
                                            const QString &layerName,
                                            QgsVectorFileWriter::ActionOnExistingFile action,
                                            QString *newLayer = nullptr,
-                                           QgsCoordinateTransformContext transformContext = QgsCoordinateTransformContext(),
-                                           QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags()
+                                           const QgsCoordinateTransformContext &transformContext = QgsCoordinateTransformContext(),
+                                           QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(),
+                                           FieldNameSource fieldNameSource = Original,
+                                           bool includeConstraints = false
                                          ) SIP_SKIP;
 
     //! QgsVectorFileWriter cannot be copied.
@@ -629,13 +679,13 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      */
     static QgsVectorFileWriter *create( const QString &fileName,
                                         const QgsFields &fields,
-                                        QgsWkbTypes::Type geometryType,
+                                        Qgis::WkbType geometryType,
                                         const QgsCoordinateReferenceSystem &srs,
                                         const QgsCoordinateTransformContext &transformContext,
                                         const QgsVectorFileWriter::SaveVectorOptions &options,
                                         QgsFeatureSink::SinkFlags sinkFlags = QgsFeatureSink::SinkFlags(),
                                         QString *newFilename = nullptr,
-                                        QString *newLayer = nullptr );
+                                        QString *newLayer = nullptr ) SIP_FACTORY;
 
     /**
      * Writes a layer out to a vector file.
@@ -647,15 +697,35 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * \param newLayer potentially modified layer name (output parameter)
      * \param errorMessage will be set to the error message text, if an error occurs while writing the layer
      * \returns Error message code, or QgsVectorFileWriter.NoError if the write operation was successful
-     * \since QGIS 3.10.3
+     * \deprecated since QGIS 3.20, use writeAsVectorFormatV3 instead
      */
-    static QgsVectorFileWriter::WriterError writeAsVectorFormatV2( QgsVectorLayer *layer,
+    Q_DECL_DEPRECATED static QgsVectorFileWriter::WriterError writeAsVectorFormatV2( QgsVectorLayer *layer,
         const QString &fileName,
         const QgsCoordinateTransformContext &transformContext,
         const QgsVectorFileWriter::SaveVectorOptions &options,
         QString *newFilename = nullptr,
         QString *newLayer = nullptr,
-        QString *errorMessage SIP_OUT = nullptr );
+        QString *errorMessage SIP_OUT = nullptr ) SIP_DEPRECATED;
+
+    /**
+     * Writes a layer out to a vector file.
+     * \param layer source layer to write
+     * \param fileName file name to write to
+     * \param transformContext coordinate transform context
+     * \param options save options
+     * \param newFilename potentially modified file name (output parameter)
+     * \param newLayer potentially modified layer name (output parameter)
+     * \param errorMessage will be set to the error message text, if an error occurs while writing the layer
+     * \returns Error message code, or QgsVectorFileWriter.NoError if the write operation was successful
+     * \since QGIS 3.20
+     */
+    static QgsVectorFileWriter::WriterError writeAsVectorFormatV3( QgsVectorLayer *layer,
+        const QString &fileName,
+        const QgsCoordinateTransformContext &transformContext,
+        const QgsVectorFileWriter::SaveVectorOptions &options,
+        QString *errorMessage SIP_OUT = nullptr,
+        QString *newFilename SIP_OUT = nullptr,
+        QString *newLayer SIP_OUT = nullptr );
 
     /**
      * Details of available filters and formats.
@@ -755,22 +825,46 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     static QString convertCodecNameForEncodingOption( const QString &codecName );
 
     //! Checks whether there were any errors in constructor
-    QgsVectorFileWriter::WriterError hasError();
+    QgsVectorFileWriter::WriterError hasError() const;
 
     //! Retrieves error message
-    QString errorMessage();
+    QString errorMessage() const;
+
+    /**
+     * Returns the GDAL (short) driver name associated with the output file.
+     *
+     * \see driverLongName()
+     * \since QGIS 3.32
+     */
+    QString driver() const;
+
+    /**
+     * Returns the GDAL long driver name associated with the output file.
+     *
+     * \see driver()
+     * \since QGIS 3.32
+     */
+    QString driverLongName() const;
+
+    /**
+     * Returns the capabilities supported by the writer.
+     *
+     * \since QGIS 3.32
+     */
+    Qgis::VectorFileWriterCapabilities capabilities() const;
 
     bool addFeature( QgsFeature &feature, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
     bool addFeatures( QgsFeatureList &features, QgsFeatureSink::Flags flags = QgsFeatureSink::Flags() ) override;
+    QString lastError() const override;
 
     /**
      * Adds a \a feature to the currently opened data source, using the style from a specified \a renderer.
      * \since QGIS 3.0
      */
-    bool addFeatureWithStyle( QgsFeature &feature, QgsFeatureRenderer *renderer, QgsUnitTypes::DistanceUnit outputUnit = QgsUnitTypes::DistanceMeters );
+    bool addFeatureWithStyle( QgsFeature &feature, QgsFeatureRenderer *renderer, Qgis::DistanceUnit outputUnit = Qgis::DistanceUnit::Meters );
 
     //! \note not available in Python bindings
-    QMap<int, int> attrIdxToOgrIdx() { return mAttrIdxToOgrIdx; } SIP_SKIP
+    QMap<int, int> attrIdxToOgrIdx() const { return mAttrIdxToOgrIdx; } SIP_SKIP
 
     //! Close opened shapefile for writing
     ~QgsVectorFileWriter() override;
@@ -782,8 +876,19 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      */
     static bool deleteShapeFile( const QString &fileName );
 
-    QgsVectorFileWriter::SymbologyExport symbologyExport() const { return mSymbologyExport; }
-    void setSymbologyExport( QgsVectorFileWriter::SymbologyExport symExport ) { mSymbologyExport = symExport; }
+    /**
+     * Returns the feature symbology export handling for the writer.
+     *
+     * \see setSymbologyExport()
+     */
+    Qgis::FeatureSymbologyExport symbologyExport() const { return mSymbologyExport; }
+
+    /**
+     * Sets the feature symbology export handling for the writer.
+     *
+     * \see symbologyExport()
+     */
+    void setSymbologyExport( Qgis::FeatureSymbologyExport symExport ) { mSymbologyExport = symExport; }
 
     /**
      * Returns the reference scale for output.
@@ -825,7 +930,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
      * Will drop M values and convert Z to 2.5D where required.
      * \note not available in Python bindings
      */
-    static OGRwkbGeometryType ogrTypeFromWkbType( QgsWkbTypes::Type type ) SIP_SKIP;
+    static OGRwkbGeometryType ogrTypeFromWkbType( Qgis::WkbType type ) SIP_SKIP;
 
     /**
      * Returns edition capabilities for an existing dataset name.
@@ -851,7 +956,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
   protected:
     //! \note not available in Python bindings
-    OGRGeometryH createEmptyGeometry( QgsWkbTypes::Type wkbType ) SIP_SKIP;
+    OGRGeometryH createEmptyGeometry( Qgis::WkbType wkbType ) SIP_SKIP;
 
     gdal::ogr_datasource_unique_ptr mDS;
     OGRLayerH mLayer = nullptr;
@@ -866,12 +971,12 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     QTextCodec *mCodec = nullptr;
 
     //! Geometry type which is being used
-    QgsWkbTypes::Type mWkbType;
+    Qgis::WkbType mWkbType;
 
     //! Map attribute indizes to OGR field indexes
     QMap<int, int> mAttrIdxToOgrIdx;
 
-    SymbologyExport mSymbologyExport;
+    Qgis::FeatureSymbologyExport mSymbologyExport = Qgis::FeatureSymbologyExport::NoSymbology;
 
     QMap< QgsSymbolLayer *, QString > mSymbolLayerTable;
 
@@ -879,10 +984,14 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     double mSymbologyScale;
 
     QString mOgrDriverName;
+    QString mOgrDriverLongName;
 
     //! Field value converter
     FieldValueConverter *mFieldValueConverter = nullptr;
 
+    //! Whether to transfer field constraints to output
+    bool mIncludeConstraints = false;
+
   private:
 #ifdef SIP_RUN
     QgsVectorFileWriter( const QgsVectorFileWriter &rh );
@@ -892,10 +1001,10 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     {
       std::unique_ptr< QgsFeatureRenderer > renderer;
       QgsCoordinateReferenceSystem sourceCrs;
-      QgsWkbTypes::Type sourceWkbType = QgsWkbTypes::Unknown;
+      Qgis::WkbType sourceWkbType = Qgis::WkbType::Unknown;
       QgsFields sourceFields;
       QString providerType;
-      long featureCount = 0;
+      long long featureCount = 0;
       QgsFeatureIds selectedFeatureIds;
       QString dataSourceUri;
       QString storageType;
@@ -905,7 +1014,7 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
       QgsRenderContext renderContext;
       bool shallTransform = false;
       QgsCoordinateReferenceSystem outputCrs;
-      QgsWkbTypes::Type destWkbType = QgsWkbTypes::Unknown;
+      Qgis::WkbType destWkbType = Qgis::WkbType::Unknown;
       QgsAttributeList attributes;
       QgsFields outputFields;
       QgsFeatureIterator sourceFeatureIterator;
@@ -956,13 +1065,14 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
         QString *newLayer = nullptr ) SIP_DEPRECATED;
 
     void init( QString vectorFileName, QString fileEncoding, const QgsFields &fields,
-               QgsWkbTypes::Type geometryType, QgsCoordinateReferenceSystem srs,
+               Qgis::WkbType geometryType, QgsCoordinateReferenceSystem srs,
                const QString &driverName, QStringList datasourceOptions,
                QStringList layerOptions, QString *newFilename,
                QgsVectorFileWriter::FieldValueConverter *fieldValueConverter,
                const QString &layerName,
                QgsVectorFileWriter::ActionOnExistingFile action, QString *newLayer, QgsFeatureSink::SinkFlags sinkFlags,
-               const QgsCoordinateTransformContext &transformContext );
+               const QgsCoordinateTransformContext &transformContext,
+               FieldNameSource fieldNameSource );
     void resetMap( const QgsAttributeList &attributes );
 
     std::unique_ptr< QgsFeatureRenderer > mRenderer;
@@ -972,7 +1082,9 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
     std::unique_ptr< QgsCoordinateTransform > mCoordinateTransform;
 
     bool mUsingTransaction = false;
-    bool supportsStringList = false;
+    QSet< QVariant::Type > mSupportedListSubTypes;
+
+    Qgis::VectorFileWriterCapabilities mCapabilities;
 
     void createSymbolLayerTable( QgsVectorLayer *vl, const QgsCoordinateTransform &ct, OGRDataSourceH ds );
     gdal::ogr_feature_unique_ptr createFeature( const QgsFeature &feature );
@@ -980,8 +1092,8 @@ class CORE_EXPORT QgsVectorFileWriter : public QgsFeatureSink
 
     //! Writes features considering symbol level order
     QgsVectorFileWriter::WriterError exportFeaturesSymbolLevels( const PreparedWriterDetails &details, QgsFeatureIterator &fit, const QgsCoordinateTransform &ct, QString *errorMessage = nullptr );
-    double mmScaleFactor( double scale, QgsUnitTypes::RenderUnit symbolUnits, QgsUnitTypes::DistanceUnit mapUnits );
-    double mapUnitScaleFactor( double scale, QgsUnitTypes::RenderUnit symbolUnits, QgsUnitTypes::DistanceUnit mapUnits );
+    double mmScaleFactor( double scale, Qgis::RenderUnit symbolUnits, Qgis::DistanceUnit mapUnits );
+    double mapUnitScaleFactor( double scale, Qgis::RenderUnit symbolUnits, Qgis::DistanceUnit mapUnits );
 
     void startRender( QgsFeatureRenderer *sourceRenderer, const QgsFields &fields );
     void stopRender();
