@@ -268,6 +268,20 @@ class CORE_EXPORT QgsVectorLayerEditUtils
      */
     int addTopologicalPoints( const QgsPointSequence &ps );
 
+    /**
+     * Merge features into a single one.
+     * \param targetFeatureId id of the target feature (will be updated)
+     * \param mergeFeatureIds id list of features to merge (will be deleted)
+     * \param mergeAttributes are the resulting attributes in the merged feature
+     * \param unionGeometry is the resulting geometry of the merged feature
+     * \param errorMessage will be set to a descriptive error message if any occurs
+     *
+     * \returns TRUE if the merge was successful, or FALSE if the operation failed.
+     *
+     * \since QGIS 3.30
+     */
+    bool mergeFeatures( const QgsFeatureId &targetFeatureId, const QgsFeatureIds &mergeFeatureIds, const QgsAttributes &mergeAttributes, const QgsGeometry &unionGeometry, QString &errorMessage SIP_OUT );
+
   private:
 
     /**
