@@ -30,7 +30,7 @@ from functools import partial
 from operator import itemgetter
 from pathlib import Path
 
-from qgis.core import Qgis, QgsApplication, QgsBlockingNetworkRequest, QgsFileUtils, QgsSettings
+from qgis.core import Qgis, QgsApplication, QgsBlockingNetworkRequest, QgsSettings
 from qgis.gui import QgsCodeEditorPython, QgsMessageBar
 from qgis.PyQt.Qsci import QsciScintilla
 from qgis.PyQt.QtCore import QByteArray, QCoreApplication, QDir, QEvent, QFileInfo, QJsonDocument, QSize, Qt, QUrl
@@ -52,7 +52,7 @@ from qgis.PyQt.QtWidgets import (
     QTreeWidgetItem,
     QWidget,
 )
-from qgis.utils import OverrideCursor
+from qgis.utils import OverrideCursor, iface
 
 
 class Editor(QgsCodeEditorPython):
@@ -68,11 +68,11 @@ class Editor(QgsCodeEditorPython):
         self.setMinimumHeight(120)
         self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
 
-        # Disable command key
+        # Disable default scintilla shortcuts
         ctrl, shift = self.SCMOD_CTRL << 16, self.SCMOD_SHIFT << 16
-        self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('L') + ctrl)
-        self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('T') + ctrl)
-        self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('D') + ctrl)
+        self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('T') + ctrl)  # Switch current line with the next one
+        self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('D') + ctrl)  # Duplicate current line / selection
+        self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('L') + ctrl)  # Delete current line
         self.SendScintilla(QsciScintilla.SCI_CLEARCMDKEY, ord('L') + ctrl + shift)
 
         # New QShortcut = ctrl+space/ctrl+alt+space for Autocomplete
@@ -112,7 +112,7 @@ class Editor(QgsCodeEditorPython):
                                      QCoreApplication.translate("PythonConsole", "Run Selected"),
                                      self.runSelectedCode, 'Ctrl+E')  # spellok
         pyQGISHelpAction = menu.addAction(QgsApplication.getThemeIcon("console/iconHelpConsole.svg"),
-                                          QCoreApplication.translate("PythonConsole", "Search Selected in PyQGIS docs"),
+                                          QCoreApplication.translate("PythonConsole", "Search Selection in PyQGIS Documentation"),
                                           self.searchSelectedTextInPyQGISDocs)
         menu.addAction(QgsApplication.getThemeIcon("mActionStart.svg"),
                        QCoreApplication.translate("PythonConsole", "Run Script"),
@@ -218,7 +218,7 @@ class Editor(QgsCodeEditorPython):
                 if showMessage:
                     msgText = QCoreApplication.translate('PythonConsole',
                                                          '<b>"{0}"</b> was not found.').format(text)
-                    self.pythonconsole.callWidgetMessageBarEditor(msgText, 0, True)
+                    self.parent.showMessage(msgText)
             else:
                 styleError = ''
             self.pythonconsole.lineEditFind.setStyleSheet(styleError)
@@ -243,7 +243,7 @@ class Editor(QgsCodeEditorPython):
         if not ACCESS_TOKEN:
             msg_text = QCoreApplication.translate(
                 'PythonConsole', 'GitHub personal access token must be generated (see Console Options)')
-            self.pythonconsole.callWidgetMessageBarEditor(msg_text, 1, True)
+            self.parent.showMessage(msg_text, Qgis.Warning, 5)
             return
 
         URL = "https://api.github.com/gists"
@@ -270,10 +270,10 @@ class Editor(QgsCodeEditorPython):
             link = _json.object()['html_url'].toString()
             QApplication.clipboard().setText(link)
             msg = QCoreApplication.translate('PythonConsole', 'URL copied to clipboard.')
-            self.pythonconsole.callWidgetMessageBarEditor(msg, 0, True)
+            self.parent.showMessage(msg)
         else:
             msg = QCoreApplication.translate('PythonConsole', 'Connection error: ')
-            self.pythonconsole.callWidgetMessageBarEditor(msg + request.erroMessage(), 0, True)
+            self.parent.showMessage(msg + request.erroMessage(), Qgis.Warning, 5)
 
     def hideEditor(self):
         self.pythonconsole.splitterObj.hide()
@@ -314,7 +314,7 @@ class Editor(QgsCodeEditorPython):
                                                     'Hey, type something to run!')
         if filename is None:
             if not self.isModified():
-                self.pythonconsole.callWidgetMessageBarEditor(msgEditorBlank, 0, True)
+                self.parent.showMessage(msgEditorBlank)
                 return
 
         deleteTempFile = False
@@ -326,8 +326,8 @@ class Editor(QgsCodeEditorPython):
                 filename = self.createTempFile()
                 deleteTempFile = True
 
-            self.pythonconsole.shell.runCommand("exec(Path('{0}').read_text())"
-                                                .format(filename.replace("\\", "/")))
+            self.pythonconsole.shell.runFile(filename)
+
             if deleteTempFile:
                 Path(filename).unlink()
 
@@ -376,7 +376,7 @@ class Editor(QgsCodeEditorPython):
             if not QFileInfo(self.path).exists():
                 msgText = QCoreApplication.translate('PythonConsole',
                                                      'The file <b>"{0}"</b> has been deleted or is not accessible').format(self.path)
-                self.pythonconsole.callWidgetMessageBarEditor(msgText, 2, False)
+                self.parent.showMessage(msgText, Qgis.Critical)
                 return
         if self.path and self.lastModified != QFileInfo(self.path).lastModified():
             self.beginUndoAction()
@@ -394,7 +394,7 @@ class Editor(QgsCodeEditorPython):
         tabWidget = self.tabwidget.currentWidget()
         msgText = QCoreApplication.translate('PythonConsole',
                                              'The file <b>"{0}"</b> is read only, please save to different file first.').format(tabWidget.path)
-        self.pythonconsole.callWidgetMessageBarEditor(msgText, 1, False)
+        self.parent.showMessage(msgText, Qgis.Warning)
 
     def loadFile(self, filename, readOnly=False):
         self.lastModified = QFileInfo(filename).lastModified()
@@ -407,6 +407,10 @@ class Editor(QgsCodeEditorPython):
     def save(self, filename=None):
         if self.isReadOnly():
             return
+
+        if self.pythonconsole.settings.value("pythonConsole/formatOnSave", False, type=bool):
+            self.reformatCode()
+
         tabwidget = self.tabwidget
         index = tabwidget.indexOf(self.parent)
         if filename:
@@ -426,7 +430,7 @@ class Editor(QgsCodeEditorPython):
 
             msgText = QCoreApplication.translate('PythonConsole',
                                                  'Script was correctly saved.')
-            self.pythonconsole.callWidgetMessageBarEditor(msgText, 0, True)
+            self.parent.showMessage(msgText)
 
         # Save the new contents
         # Need to use newline='' to avoid adding extra \r characters on Windows
@@ -487,6 +491,9 @@ class Editor(QgsCodeEditorPython):
 
         super().keyPressEvent(e)
 
+    def showMessage(self, title, text, level):
+        self.parent.showMessage(text, level, title=title)
+
 
 class EditorTab(QWidget):
 
@@ -536,6 +543,9 @@ class EditorTab(QWidget):
         except AttributeError:
             return setattr(self._editor, name, value)
 
+    def showMessage(self, text, level=Qgis.Info, timeout=-1, title=""):
+        self.infoBar.pushMessage(title, text, level, timeout)
+
 
 class EditorTabWidget(QTabWidget):
 
@@ -918,11 +928,6 @@ class EditorTabWidget(QTabWidget):
         if tabWidget and tabWidget.path:
             self.settings.setValue("pythonConsole/lastDirPath", tabWidget.path)
 
-    def widgetMessageBar(self, iface, text, level, timed=True):
-        messageLevel = [Qgis.Info, Qgis.Warning, Qgis.Critical]
-        if timed:
-            timeout = iface.messageTimeout()
-        else:
-            timeout = 0
+    def showMessage(self, text, level=Qgis.Info, timeout=-1, title=""):
         currWidget = self.currentWidget()
-        currWidget.infoBar.pushMessage(text, messageLevel[level], timeout)
+        currWidget.showMessage(text, level, timeout, title)
