@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     ClipVectorByMask.py
@@ -84,20 +82,21 @@ class ClipVectorByMask(GdalAlgorithm):
 
         output, outputFormat = GdalUtils.ogrConnectionStringAndFormat(outFile, context)
 
-        arguments = []
-        arguments.append('-clipsrc')
-        arguments.append(maskLayer)
-        arguments.append('-clipsrclayer')
-        arguments.append(maskLayerName)
+        arguments = [
+            '-clipsrc',
+            maskLayer,
+            '-clipsrclayer',
+            maskLayerName,
 
-        arguments.append(output)
-        arguments.append(inLayer)
-        arguments.append(inLayerName)
+            output,
+            inLayer,
+            inLayerName,
+        ]
 
         if options:
             arguments.append(options)
 
         if outputFormat:
-            arguments.append('-f {}'.format(outputFormat))
+            arguments.append(f'-f {outputFormat}')
 
         return [self.commandName(), GdalUtils.escapeAndJoin(arguments)]
