@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     fillnodata.py
@@ -41,7 +39,6 @@ pluginPath = os.path.split(os.path.split(os.path.dirname(__file__))[0])[0]
 
 
 class fillnodata(GdalAlgorithm):
-
     INPUT = 'INPUT'
     BAND = 'BAND'
     DISTANCE = 'DISTANCE'
@@ -116,9 +113,19 @@ class fillnodata(GdalAlgorithm):
         return super().flags() | QgsProcessingAlgorithm.FlagDisplayNameIsLiteral
 
     def getConsoleCommands(self, parameters, context, feedback, executing=True):
-        arguments = []
-        arguments.append('-md')
-        arguments.append(str(self.parameterAsInt(parameters, self.DISTANCE, context)))
+        raster = self.parameterAsRasterLayer(parameters, self.INPUT, context)
+        if raster is None:
+            raise QgsProcessingException(self.invalidRasterError(parameters, self.INPUT))
+
+        out = self.parameterAsOutputLayer(parameters, self.OUTPUT, context)
+        self.setOutputValue(self.OUTPUT, out)
+
+        arguments = [
+            raster.source(),
+            out,
+            '-md',
+            str(self.parameterAsInt(parameters, self.DISTANCE, context)),
+        ]
 
         nIterations = self.parameterAsInt(parameters, self.ITERATIONS, context)
         if nIterations:
@@ -133,33 +140,23 @@ class fillnodata(GdalAlgorithm):
 
         mask = self.parameterAsRasterLayer(parameters, self.MASK_LAYER, context)
         if mask:
-            arguments.append('-mask {}'.format(mask.source()))
+            arguments.append('-mask')
+            arguments.append(mask.source())
+
+        output_format = QgsRasterFileWriter.driverForExtension(os.path.splitext(out)[1])
+        if not output_format:
+            raise QgsProcessingException(self.tr('Output format is invalid'))
 
-        out = self.parameterAsOutputLayer(parameters, self.OUTPUT, context)
-        self.setOutputValue(self.OUTPUT, out)
         arguments.append('-of')
-        arguments.append(QgsRasterFileWriter.driverForExtension(os.path.splitext(out)[1]))
-
-        raster = self.parameterAsRasterLayer(parameters, self.INPUT, context)
-        if raster is None:
-            raise QgsProcessingException(self.invalidRasterError(parameters, self.INPUT))
-
-        options = self.parameterAsString(parameters, self.OPTIONS, context)
-        if options:
-            arguments.extend(GdalUtils.parseCreationOptions(options))
+        arguments.append(output_format)
 
         if self.EXTRA in parameters and parameters[self.EXTRA] not in (None, ''):
             extra = self.parameterAsString(parameters, self.EXTRA, context)
             arguments.append(extra)
 
-        arguments.append(raster.source())
-        arguments.append(out)
+        # Until https://github.com/OSGeo/gdal/issues/7651 is fixed, creation options should be latest argument
+        options = self.parameterAsString(parameters, self.OPTIONS, context)
+        if options:
+            arguments.extend(GdalUtils.parseCreationOptions(options))
 
-        if isWindows():
-            commands = ["python3", "-m", self.commandName()]
-        else:
-            commands = [self.commandName() + '.py']
-
-        commands.append(GdalUtils.escapeAndJoin(arguments))
-
-        return commands
+        return [self.commandName() + ('.bat' if isWindows() else '.py'), GdalUtils.escapeAndJoin(arguments)]
