@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     BatchInputSelectionPanel.py
@@ -38,20 +36,20 @@ from qgis.core import (QgsMapLayer,
                        QgsProcessingParameterRasterLayer,
                        QgsProcessingParameterVectorLayer,
                        QgsProcessingParameterMeshLayer,
-                       QgsProcessingParameterFeatureSource)
+                       QgsProcessingParameterPointCloudLayer,
+                       QgsProcessingParameterFeatureSource,
+                       QgsProcessingParameterMapLayer)
 
 from processing.gui.MultipleInputDialog import MultipleInputDialog
 
-from processing.gui.ParameterGuiUtils import getFileFilter
 from processing.tools import dataobjects
 
 
 class BatchInputSelectionPanel(QWidget):
-
     valueChanged = pyqtSignal()
 
     def __init__(self, param, row, col, dialog):
-        super(BatchInputSelectionPanel, self).__init__(None)
+        super().__init__(None)
         self.param = param
         self.dialog = dialog
         self.row = row
@@ -109,10 +107,16 @@ class BatchInputSelectionPanel(QWidget):
             layers = QgsProcessingUtils.compatibleRasterLayers(QgsProject.instance())
         elif isinstance(self.param, QgsProcessingParameterVectorLayer):
             layers = QgsProcessingUtils.compatibleVectorLayers(QgsProject.instance())
+        elif isinstance(self.param, QgsProcessingParameterMapLayer):
+            layers = QgsProcessingUtils.compatibleLayers(QgsProject.instance())
         elif (isinstance(self.param, QgsProcessingParameterMeshLayer)
               or (isinstance(self.param, QgsProcessingParameterMultipleLayers) and
                   self.param.layerType() == QgsProcessing.TypeMesh)):
             layers = QgsProcessingUtils.compatibleMeshLayers(QgsProject.instance())
+        elif (isinstance(self.param, QgsProcessingParameterPointCloudLayer)
+              or (isinstance(self.param, QgsProcessingParameterMultipleLayers) and
+                  self.param.layerType() == QgsProcessing.TypePointCloud)):
+            layers = QgsProcessingUtils.compatiblePointCloudLayers(QgsProject.instance())
         else:
             datatypes = [QgsProcessing.TypeVectorAnyGeometry]
             if isinstance(self.param, QgsProcessingParameterFeatureSource):
@@ -171,7 +175,7 @@ class BatchInputSelectionPanel(QWidget):
 
         if not seldir:
             ret, selected_filter = QFileDialog.getOpenFileNames(
-                self, self.tr('Select Files'), path, getFileFilter(self.param)
+                self, self.tr('Select Files'), path, self.param.createFileFilter()
             )
         else:
             ret = QFileDialog.getExistingDirectory(self, self.tr('Select Directory'), path)
