@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     BatchPanel.py
@@ -24,6 +22,7 @@ __copyright__ = '(C) 2014, Alexander Bruy'
 import os
 import json
 import warnings
+from pathlib import Path
 
 from qgis.PyQt import uic
 from qgis.PyQt.QtWidgets import (
@@ -36,7 +35,19 @@ from qgis.PyQt.QtWidgets import (
     QMenu,
     QAction
 )
-from qgis.PyQt.QtGui import QPalette
+
+# adding to this list? also update the QgsProcessingHistoryProvider executeAlgorithm imports!!
+
+from qgis.PyQt.QtCore import (
+    QTime,  # NOQA - must be here for saved file evaluation
+    QDate,  # NOQA - must be here for saved file evaluation
+    QDateTime  # NOQA - must be here for saved file evaluation
+)
+from qgis.PyQt.QtGui import (
+    QPalette,
+    QColor,  # NOQA - must be here for saved file evaluation
+)
+
 from qgis.PyQt.QtCore import (
     QDir,
     QFileInfo,
@@ -48,6 +59,7 @@ from qgis.core import (
     QgsSettings,
     QgsProperty,  # NOQA - must be here for saved file evaluation
     QgsProject,
+    QgsFeatureRequest,  # NOQA - must be here for saved file evaluation
     QgsProcessingFeatureSourceDefinition,  # NOQA - must be here for saved file evaluation
     QgsCoordinateReferenceSystem,  # NOQA - must be here for saved file evaluation
     QgsProcessingParameterDefinition,
@@ -56,20 +68,27 @@ from qgis.core import (
     QgsProcessingParameterMapLayer,
     QgsProcessingParameterRasterLayer,
     QgsProcessingParameterMeshLayer,
+    QgsProcessingParameterPointCloudLayer,
     QgsProcessingParameterVectorLayer,
     QgsProcessingParameterFeatureSource,
     QgsProcessingParameterRasterDestination,
     QgsProcessingParameterVectorDestination,
+    QgsProcessingParameterMultipleLayers,
     QgsProcessingParameterFeatureSink,
     QgsProcessingOutputLayerDefinition,
     QgsExpressionContextUtils,
-    QgsExpression
+    QgsProcessing,
+    QgsExpression,
+    QgsRasterLayer,
+    QgsProcessingUtils,
+    QgsFileFilterGenerator
 )
 from qgis.gui import (
     QgsProcessingParameterWidgetContext,
     QgsProcessingContextGenerator,
     QgsFindFilesByPatternDialog,
-    QgsExpressionBuilderDialog
+    QgsExpressionBuilderDialog,
+    QgsPanelWidget
 )
 from qgis.utils import iface
 
@@ -78,6 +97,7 @@ from processing.gui.BatchOutputSelectionPanel import BatchOutputSelectionPanel
 
 from processing.tools import dataobjects
 from processing.tools.dataobjects import createContext
+from processing.gui.MultipleInputDialog import MultipleInputDialog
 
 pluginPath = os.path.split(os.path.dirname(__file__))[0]
 
@@ -132,12 +152,7 @@ class BatchPanelFillWidget(QToolButton):
         add_by_expression.setToolTip(self.tr('Adds new parameter values by evaluating an expression'))
         self.menu.addAction(add_by_expression)
 
-        if isinstance(self.parameterDefinition, (QgsProcessingParameterFile,
-                                                 QgsProcessingParameterMapLayer,
-                                                 QgsProcessingParameterRasterLayer,
-                                                 QgsProcessingParameterMeshLayer,
-                                                 QgsProcessingParameterVectorLayer,
-                                                 QgsProcessingParameterFeatureSource)):
+        if not self.parameterDefinition.isDestination() and isinstance(self.parameterDefinition, QgsFileFilterGenerator):
             self.menu.addSeparator()
             find_by_pattern_action = QAction(QCoreApplication.translate('BatchPanel', 'Add Files by Pattern…'),
                                              self.menu)
@@ -145,6 +160,35 @@ class BatchPanelFillWidget(QToolButton):
             find_by_pattern_action.setToolTip(self.tr('Adds files by a file pattern match'))
             self.menu.addAction(find_by_pattern_action)
 
+            select_file_action = QAction(
+                QCoreApplication.translate('BatchInputSelectionPanel', 'Select Files…'), self.menu)
+            select_file_action.triggered.connect(self.showFileSelectionDialog)
+            self.menu.addAction(select_file_action)
+
+            select_directory_action = QAction(
+                QCoreApplication.translate('BatchInputSelectionPanel', 'Add All Files from a Directory…'), self.menu)
+            select_directory_action.triggered.connect(self.showDirectorySelectionDialog)
+            self.menu.addAction(select_directory_action)
+
+            if not isinstance(self.parameterDefinition, QgsProcessingParameterFile):
+                select_layer_action = QAction(
+                    QCoreApplication.translate('BatchInputSelectionPanel', 'Select from Open Layers…'), self.menu)
+                select_layer_action.triggered.connect(self.showLayerSelectionDialog)
+                self.menu.addAction(select_layer_action)
+
+    def findStartingRow(self):
+        first_row = 0
+        for row in range(self.panel.batchRowCount()):
+            wrapper = self.panel.wrappers[row][self.column]
+            if wrapper is None:
+                break
+            else:
+                value = wrapper.parameterValue()
+                if value is None:
+                    break
+                first_row += 1
+        return first_row
+
     def fillDown(self):
         """
         Copy the top value down
@@ -186,9 +230,123 @@ class BatchPanelFillWidget(QToolButton):
             files = dlg.files()
             context = dataobjects.createContext()
 
-            first_row = self.panel.batchRowCount() if self.panel.batchRowCount() > 1 else 0
+            first_row = self.findStartingRow()
+            self.panel.tblParameters.setUpdatesEnabled(False)
             for row, file in enumerate(files):
                 self.setRowValue(first_row + row, file, context)
+            self.panel.tblParameters.setUpdatesEnabled(True)
+
+    def showFileSelectionDialog(self):
+        settings = QgsSettings()
+        if settings.contains('/Processing/LastInputPath'):
+            path = str(settings.value('/Processing/LastInputPath'))
+        else:
+            path = QDir.homePath()
+
+        files, selected_filter = QFileDialog.getOpenFileNames(
+            self, self.tr('Select Files'), path, self.parameterDefinition.createFileFilter()
+        )
+
+        if not files:
+            return
+
+        settings.setValue('/Processing/LastInputPath', os.path.dirname(str(files[0])))
+
+        context = dataobjects.createContext()
+
+        first_row = self.findStartingRow()
+        self.panel.tblParameters.setUpdatesEnabled(False)
+        for row, file in enumerate(files):
+            self.setRowValue(first_row + row, file, context)
+        self.panel.tblParameters.setUpdatesEnabled(True)
+
+    def showDirectorySelectionDialog(self):
+        settings = QgsSettings()
+        if settings.contains('/Processing/LastInputPath'):
+            path = str(settings.value('/Processing/LastInputPath'))
+        else:
+            path = QDir.homePath()
+
+        folder = QFileDialog.getExistingDirectory(self, self.tr('Select Directory'), path)
+        if not folder:
+            return
+
+        settings.setValue('/Processing/LastInputPath', folder)
+
+        files = []
+        for pp in Path(folder).rglob("*"):
+            if not pp.is_file():
+                continue
+
+            p = pp.as_posix()
+
+            if isinstance(self.parameterDefinition, QgsProcessingParameterRasterLayer) or (
+                isinstance(self.parameterDefinition, QgsProcessingParameterMultipleLayers)
+                and self.parameterDefinition.layerType() == QgsProcessing.TypeRaster
+            ):
+                if not QgsRasterLayer.isValidRasterFileName(p):
+                    continue
+
+            files.append(p)
+
+        if not files:
+            return
+
+        context = dataobjects.createContext()
+
+        first_row = self.findStartingRow()
+        self.panel.tblParameters.setUpdatesEnabled(False)
+        for row, file in enumerate(files):
+            self.setRowValue(first_row + row, file, context)
+        self.panel.tblParameters.setUpdatesEnabled(True)
+
+    def showLayerSelectionDialog(self):
+        layers = []
+        if isinstance(self.parameterDefinition, QgsProcessingParameterRasterLayer):
+            layers = QgsProcessingUtils.compatibleRasterLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition,
+                        QgsProcessingParameterMultipleLayers) and self.parameterDefinition.layerType() == QgsProcessing.TypeRaster:
+            layers = QgsProcessingUtils.compatibleRasterLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition, QgsProcessingParameterVectorLayer):
+            layers = QgsProcessingUtils.compatibleVectorLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition, QgsProcessingParameterMapLayer):
+            layers = QgsProcessingUtils.compatibleLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition, QgsProcessingParameterMeshLayer):
+            layers = QgsProcessingUtils.compatibleMeshLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition,
+                        QgsProcessingParameterMultipleLayers) and self.parameterDefinition.layerType() == QgsProcessing.TypeMesh:
+            layers = QgsProcessingUtils.compatibleMeshLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition, QgsProcessingParameterPointCloudLayer):
+            layers = QgsProcessingUtils.compatiblePointCloudLayers(QgsProject.instance())
+        elif isinstance(self.parameterDefinition,
+                        QgsProcessingParameterMultipleLayers) and self.parameterDefinition.layerType() == QgsProcessing.TypePointCloud:
+            layers = QgsProcessingUtils.compatiblePointCloudLayers(QgsProject.instance())
+        else:
+            datatypes = [QgsProcessing.TypeVectorAnyGeometry]
+            if isinstance(self.parameterDefinition, QgsProcessingParameterFeatureSource):
+                datatypes = self.parameterDefinition.dataTypes()
+            elif isinstance(self.parameterDefinition, QgsProcessingParameterMultipleLayers):
+                datatypes = [self.parameterDefinition.layerType()]
+
+            if QgsProcessing.TypeVectorAnyGeometry not in datatypes:
+                layers = QgsProcessingUtils.compatibleVectorLayers(QgsProject.instance(), datatypes)
+            else:
+                layers = QgsProcessingUtils.compatibleVectorLayers(QgsProject.instance())
+
+        dlg = MultipleInputDialog([layer.name() for layer in layers])
+        dlg.exec_()
+
+        if not dlg.selectedoptions:
+            return
+
+        selected = dlg.selectedoptions
+
+        context = dataobjects.createContext()
+
+        first_row = self.findStartingRow()
+        for row, selected_idx in enumerate(selected):
+            value = layers[selected_idx].id()
+            self.setRowValue(first_row + row, value, context)
 
     def calculateByExpression(self):
         """
@@ -210,7 +368,7 @@ class BatchPanelFillWidget(QToolButton):
         expression_context = context.expressionContext()
 
         # use the first row parameter values as a preview during expression creation
-        params = self.panel.parametersForRow(0, warnOnInvalid=False)
+        params, ok = self.panel.parametersForRow(0, warnOnInvalid=False)
         alg_scope = QgsExpressionContextUtils.processingAlgorithmScope(self.panel.alg, params, context)
 
         # create explicit variables corresponding to every parameter
@@ -242,12 +400,15 @@ class BatchPanelFillWidget(QToolButton):
             if type(res) is not list:
                 res = [res]
 
-            first_row = self.panel.batchRowCount() if self.panel.batchRowCount() > 1 else 0
+            first_row = self.findStartingRow()
+            self.panel.tblParameters.setUpdatesEnabled(False)
             for row, value in enumerate(res):
                 self.setRowValue(row + first_row, value, context)
+            self.panel.tblParameters.setUpdatesEnabled(True)
         else:
+            self.panel.tblParameters.setUpdatesEnabled(False)
             for row in range(self.panel.batchRowCount()):
-                params = self.panel.parametersForRow(row, warnOnInvalid=False)
+                params, ok = self.panel.parametersForRow(row, warnOnInvalid=False)
 
                 # remove previous algorithm scope -- we need to rebuild this completely, using the
                 # other parameter values from the current row
@@ -267,14 +428,15 @@ class BatchPanelFillWidget(QToolButton):
                 exp = QgsExpression(dlg.expressionText())
                 value = exp.evaluate(expression_context)
                 self.setRowValue(row, value, context)
+            self.panel.tblParameters.setUpdatesEnabled(True)
 
 
-class BatchPanel(BASE, WIDGET):
+class BatchPanel(QgsPanelWidget, WIDGET):
     PARAMETERS = "PARAMETERS"
     OUTPUTS = "OUTPUTS"
 
     def __init__(self, parent, alg):
-        super(BatchPanel, self).__init__(None)
+        super().__init__(None)
         self.setupUi(self)
 
         self.wrappers = []
@@ -291,7 +453,7 @@ class BatchPanel(BASE, WIDGET):
         self.alg = alg
         self.parent = parent
 
-        self.btnAdd.clicked.connect(self.addRow)
+        self.btnAdd.clicked.connect(lambda: self.addRow(1))
         self.btnRemove.clicked.connect(self.removeRows)
         self.btnOpen.clicked.connect(self.load)
         self.btnSave.clicked.connect(self.save)
@@ -331,7 +493,7 @@ class BatchPanel(BASE, WIDGET):
 
         # Determine column count
         self.tblParameters.setColumnCount(
-            self.alg.countVisibleParameters())
+            len(self.alg.parameterDefinitions()))
 
         # Table headers
         column = 0
@@ -340,7 +502,7 @@ class BatchPanel(BASE, WIDGET):
                 continue
             self.tblParameters.setHorizontalHeaderItem(
                 column, QTableWidgetItem(param.description()))
-            if param.flags() & QgsProcessingParameterDefinition.FlagAdvanced:
+            if param.flags() & QgsProcessingParameterDefinition.FlagAdvanced or param.flags() & QgsProcessingParameterDefinition.FlagHidden:
                 self.tblParameters.setColumnHidden(column, True)
 
             self.column_to_parameter_definition[column] = param.name()
@@ -373,8 +535,6 @@ class BatchPanel(BASE, WIDGET):
     def clear(self):
         self.tblParameters.setRowCount(1)
         self.wrappers = []
-        self.column_to_parameter_definition = {}
-        self.parameter_to_column = {}
 
     def load(self):
         context = dataobjects.createContext()
@@ -398,26 +558,24 @@ class BatchPanel(BASE, WIDGET):
                 self.addRow()
                 params = alg[self.PARAMETERS]
                 outputs = alg[self.OUTPUTS]
-                column = 0
+
                 for param in self.alg.parameterDefinitions():
-                    if param.flags() & QgsProcessingParameterDefinition.FlagHidden:
-                        continue
                     if param.isDestination():
                         continue
                     if param.name() in params:
+                        column = self.parameter_to_column[param.name()]
                         value = eval(params[param.name()])
                         wrapper = self.wrappers[row][column]
                         wrapper.setParameterValue(value, context)
-                    column += 1
 
                 for out in self.alg.destinationParameterDefinitions():
                     if out.flags() & QgsProcessingParameterDefinition.FlagHidden:
                         continue
                     if out.name() in outputs:
+                        column = self.parameter_to_column[out.name()]
                         value = outputs[out.name()].strip("'")
                         widget = self.tblParameters.cellWidget(row + 1, column)
                         widget.setValue(value)
-                    column += 1
         except TypeError:
             QMessageBox.critical(
                 self,
@@ -430,13 +588,12 @@ class BatchPanel(BASE, WIDGET):
         for row in range(self.batchRowCount()):
             algParams = {}
             algOutputs = {}
-            col = 0
             alg = self.alg
             for param in alg.parameterDefinitions():
-                if param.flags() & QgsProcessingParameterDefinition.FlagHidden:
-                    continue
                 if param.isDestination():
                     continue
+
+                col = self.parameter_to_column[param.name()]
                 wrapper = self.wrappers[row][col]
 
                 # For compatibility with 3.x API, we need to check whether the wrapper is
@@ -451,25 +608,24 @@ class BatchPanel(BASE, WIDGET):
                 value = wrapper.parameterValue()
 
                 if not param.checkValueIsAcceptable(value, context):
-                    self.parent.messageBar().pushMessage("", self.tr(
-                        'Wrong or missing parameter value: {0} (row {1})').format(
-                        param.description(), row + 1),
-                        level=Qgis.Warning, duration=5)
+                    msg = self.tr('Wrong or missing parameter value: {0} (row {1})').format(
+                        param.description(), row + 2)
+                    self.parent.messageBar().pushMessage("", msg, level=Qgis.Warning, duration=5)
                     return
                 algParams[param.name()] = param.valueAsPythonString(value, context)
-                col += 1
+
             for out in alg.destinationParameterDefinitions():
                 if out.flags() & QgsProcessingParameterDefinition.FlagHidden:
                     continue
+                col = self.parameter_to_column[out.name()]
                 widget = self.tblParameters.cellWidget(row + 1, col)
                 text = widget.getValue()
                 if text.strip() != '':
                     algOutputs[out.name()] = text.strip()
-                    col += 1
                 else:
                     self.parent.messageBar().pushMessage("",
                                                          self.tr('Wrong or missing output value: {0} (row {1})').format(
-                                                             out.description(), row + 1),
+                                                             out.description(), row + 2),
                                                          level=Qgis.Warning, duration=5)
                     return
             toSave.append({self.PARAMETERS: algParams, self.OUTPUTS: algOutputs})
@@ -494,6 +650,7 @@ class BatchPanel(BASE, WIDGET):
         widget_context = QgsProcessingParameterWidgetContext()
         widget_context.setProject(QgsProject.instance())
         if iface is not None:
+            widget_context.setActiveLayer(iface.activeLayer())
             widget_context.setMapCanvas(iface.mapCanvas())
 
         widget_context.setMessageBar(self.parent.messageBar())
@@ -521,42 +678,39 @@ class BatchPanel(BASE, WIDGET):
             param_definition = self.alg.parameterDefinition(self.column_to_parameter_definition[col])
             self.tblParameters.setCellWidget(0, col, BatchPanelFillWidget(param_definition, col, self))
 
-    def addRow(self):
-        self.wrappers.append([None] * self.tblParameters.columnCount())
-        self.tblParameters.setRowCount(self.tblParameters.rowCount() + 1)
+    def addRow(self, nb=1):
+        self.tblParameters.setUpdatesEnabled(False)
+        self.tblParameters.setRowCount(self.tblParameters.rowCount() + nb)
 
         context = dataobjects.createContext()
 
         wrappers = {}
-        row = self.tblParameters.rowCount() - 1
-        column = 0
-        for param in self.alg.parameterDefinitions():
-            if param.flags() & QgsProcessingParameterDefinition.FlagHidden or param.isDestination():
-                continue
+        row = self.tblParameters.rowCount() - nb
+        while row < self.tblParameters.rowCount():
+            self.wrappers.append([None] * self.tblParameters.columnCount())
+            for param in self.alg.parameterDefinitions():
+                if param.isDestination():
+                    continue
 
-            wrapper = WidgetWrapperFactory.create_wrapper(param, self.parent, row, column)
-            wrappers[param.name()] = wrapper
-            self.setCellWrapper(row, column, wrapper, context)
-            column += 1
+                column = self.parameter_to_column[param.name()]
+                wrapper = WidgetWrapperFactory.create_wrapper(param, self.parent, row, column)
+                wrappers[param.name()] = wrapper
+                self.setCellWrapper(row, column, wrapper, context)
 
-        for out in self.alg.destinationParameterDefinitions():
-            if out.flags() & QgsProcessingParameterDefinition.FlagHidden:
-                continue
+            for out in self.alg.destinationParameterDefinitions():
+                if out.flags() & QgsProcessingParameterDefinition.FlagHidden:
+                    continue
 
-            self.tblParameters.setCellWidget(
-                row, column, BatchOutputSelectionPanel(
-                    out, self.alg, row, column, self))
-            column += 1
+                column = self.parameter_to_column[out.name()]
+                self.tblParameters.setCellWidget(
+                    row, column, BatchOutputSelectionPanel(
+                        out, self.alg, row, column, self))
 
-        if len(self.alg.destinationParameterDefinitions()) > 0:
-            item = QComboBox()
-            item.addItem(self.tr('Yes'))
-            item.addItem(self.tr('No'))
-            item.setCurrentIndex(0)
-            self.tblParameters.setCellWidget(row, column, item)
+            for wrapper in list(wrappers.values()):
+                wrapper.postInitialize(list(wrappers.values()))
+            row += 1
 
-        for wrapper in list(wrappers.values()):
-            wrapper.postInitialize(list(wrappers.values()))
+        self.tblParameters.setUpdatesEnabled(True)
 
     def removeRows(self):
         rows = set()
@@ -572,9 +726,19 @@ class BatchPanel(BASE, WIDGET):
             del self.wrappers[row - 1]
             self.tblParameters.removeRow(row)
 
+        # resynchronize stored row numbers for table widgets
+        for row in range(1, self.tblParameters.rowCount()):
+            for col in range(0, self.tblParameters.columnCount()):
+                cell_widget = self.tblParameters.cellWidget(row, col)
+                if not cell_widget:
+                    continue
+
+                if isinstance(cell_widget, BatchOutputSelectionPanel):
+                    cell_widget.row = row
+
     def toggleAdvancedMode(self, checked):
         for param in self.alg.parameterDefinitions():
-            if param.flags() & QgsProcessingParameterDefinition.FlagAdvanced:
+            if param.flags() & QgsProcessingParameterDefinition.FlagAdvanced and not (param.flags() & QgsProcessingParameterDefinition.FlagHidden):
                 self.tblParameters.setColumnHidden(self.parameter_to_column[param.name()], not checked)
 
     def valueForParameter(self, row, parameter_name):
@@ -588,29 +752,36 @@ class BatchPanel(BASE, WIDGET):
         """
         Returns the parameters dictionary corresponding to a row in the batch table
         """
-        col = 0
         parameters = {}
         for param in self.alg.parameterDefinitions():
-            if param.flags() & QgsProcessingParameterDefinition.FlagHidden or param.isDestination():
+            if param.isDestination():
                 continue
+            col = self.parameter_to_column[param.name()]
             wrapper = self.wrappers[row][col]
             parameters[param.name()] = wrapper.parameterValue()
             if warnOnInvalid and not param.checkValueIsAcceptable(wrapper.parameterValue()):
                 self.parent.messageBar().pushMessage("",
                                                      self.tr('Wrong or missing parameter value: {0} (row {1})').format(
-                                                         param.description(), row + 1),
+                                                         param.description(), row + 2),
                                                      level=Qgis.Warning, duration=5)
-                return {}
-            col += 1
+                return {}, False
+
         count_visible_outputs = 0
         for out in self.alg.destinationParameterDefinitions():
             if out.flags() & QgsProcessingParameterDefinition.FlagHidden:
                 continue
 
+            col = self.parameter_to_column[out.name()]
+
             count_visible_outputs += 1
             widget = self.tblParameters.cellWidget(row + 1, col)
             text = widget.getValue()
             if not warnOnInvalid or out.checkValueIsAcceptable(text):
+                ok, error = out.isSupportedOutputValue(text, createContext())
+                if not ok:
+                    self.parent.messageBar().pushMessage("", error, level=Qgis.Warning, duration=5)
+                    return {}, False
+
                 if isinstance(out, (QgsProcessingParameterRasterDestination,
                                     QgsProcessingParameterVectorDestination,
                                     QgsProcessingParameterFeatureSink)):
@@ -618,10 +789,8 @@ class BatchPanel(BASE, WIDGET):
                     parameters[out.name()] = QgsProcessingOutputLayerDefinition(text, destinationProject)
                 else:
                     parameters[out.name()] = text
-                col += 1
             else:
-                self.parent.messageBar().pushMessage("", self.tr('Wrong or missing output value: {0} (row {1})').format(
-                    out.description(), row + 1),
-                    level=Qgis.Warning, duration=5)
-                return {}
-        return parameters
+                msg = self.tr('Wrong or missing output value: {0} (row {1})').format(out.description(), row + 2)
+                self.parent.messageBar().pushMessage("", msg, level=Qgis.Warning, duration=5)
+                return {}, False
+        return parameters, True
