@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     AlgorithmsTest.py
@@ -66,13 +64,13 @@ def processingTestDataPath():
     return os.path.join(os.path.dirname(__file__), 'testdata')
 
 
-class AlgorithmsTest(object):
+class AlgorithmsTest:
 
     def test_algorithms(self):
         """
         This is the main test function. All others will be executed based on the definitions in testdata/algorithm_tests.yaml
         """
-        with open(os.path.join(processingTestDataPath(), self.test_definition_file()), 'r') as stream:
+        with open(os.path.join(processingTestDataPath(), self.test_definition_file())) as stream:
             algorithm_tests = yaml.load(stream, Loader=yaml.SafeLoader)
 
         if 'tests' in algorithm_tests and algorithm_tests['tests'] is not None:
@@ -176,11 +174,11 @@ class AlgorithmsTest(object):
 
         feedback = QgsProcessingFeedback()
 
-        print('Algorithm parameters are {}'.format(parameters))
+        print(f'Algorithm parameters are {parameters}')
 
         # first check that algorithm accepts the parameters we pass...
         ok, msg = alg.checkParameterValues(parameters, context)
-        self.assertTrue(ok, 'Algorithm failed checkParameterValues with result {}'.format(msg))
+        self.assertTrue(ok, f'Algorithm failed checkParameterValues with result {msg}')
 
         if expectFailure:
             try:
@@ -192,7 +190,7 @@ class AlgorithmsTest(object):
                 pass
         else:
             results, ok = alg.run(parameters, context, feedback)
-            self.assertTrue(ok, 'params: {}, results: {}'.format(parameters, results))
+            self.assertTrue(ok, f'params: {parameters}, results: {results}')
             self.check_results(results, context, parameters, defs['results'])
 
     def load_params(self, params):
@@ -292,7 +290,7 @@ class AlgorithmsTest(object):
             self.cleanup_paths.append(tmpdir)
             path, file_name = os.path.split(filepath)
             base, ext = os.path.splitext(file_name)
-            for file in glob.glob(os.path.join(path, '{}.*'.format(base))):
+            for file in glob.glob(os.path.join(path, f'{base}.*')):
                 shutil.copy(os.path.join(path, file), tmpdir)
             filepath = os.path.join(tmpdir, file_name)
             self.in_place_layers[id] = filepath
@@ -316,7 +314,7 @@ class AlgorithmsTest(object):
             options.loadDefaultStyle = False
             lyr = QgsRasterLayer(filepath, param['name'], 'gdal', options)
 
-        self.assertTrue(lyr.isValid(), 'Could not load layer "{}" from param {}'.format(filepath, param))
+        self.assertTrue(lyr.isValid(), f'Could not load layer "{filepath}" from param {param}')
         QgsProject.instance().addMapLayer(lyr)
         return lyr
 
@@ -342,7 +340,7 @@ class AlgorithmsTest(object):
         if filepath.startswith('ogr:'):
             if not prefix[-1] == os.path.sep:
                 prefix += os.path.sep
-            filepath = re.sub(r"dbname='", "dbname='{}".format(prefix), filepath)
+            filepath = re.sub(r"dbname='", f"dbname='{prefix}", filepath)
         else:
             filepath = os.path.join(prefix, filepath)
 
@@ -371,7 +369,7 @@ class AlgorithmsTest(object):
                     try:
                         results[id]
                     except KeyError as e:
-                        raise KeyError('Expected result {} does not exist in {}'.format(str(e), list(results.keys())))
+                        raise KeyError(f'Expected result {str(e)} does not exist in {list(results.keys())}')
 
                     if isinstance(results[id], QgsMapLayer):
                         result_lyr = results[id]
@@ -401,8 +399,8 @@ class AlgorithmsTest(object):
                     self.assertTrue(res, 'Could not find matching layer in expected results')
 
             elif 'rasterhash' == expected_result['type']:
-                print("id:{} result:{}".format(id, results[id]))
-                self.assertTrue(os.path.exists(results[id]), 'File does not exist: {}, {}'.format(results[id], params))
+                print(f"id:{id} result:{results[id]}")
+                self.assertTrue(os.path.exists(results[id]), f'File does not exist: {results[id]}, {params}')
                 dataset = gdal.Open(results[id], GA_ReadOnly)
                 dataArray = nan_to_num(dataset.ReadAsArray(0))
                 strhash = hashlib.sha224(dataArray.data).hexdigest()
@@ -431,7 +429,7 @@ class AlgorithmsTest(object):
 
                 self.assertDirectoriesEqual(expected_dirpath, result_dirpath)
             elif 'regex' == expected_result['type']:
-                with open(results[id], 'r') as file:
+                with open(results[id]) as file:
                     data = file.read()
 
                 for rule in expected_result.get('rules', []):
@@ -459,9 +457,9 @@ class GenericAlgorithmsTest(unittest.TestCase):
 
     def testAlgorithmCompliance(self):
         for p in QgsApplication.processingRegistry().providers():
-            print('testing provider {}'.format(p.id()))
+            print(f'testing provider {p.id()}')
             for a in p.algorithms():
-                print('testing algorithm {}'.format(a.id()))
+                print(f'testing algorithm {a.id()}')
                 self.check_algorithm(a)
 
     def check_algorithm(self, alg):
